import React, { useEffect, useState } from 'react'

import { Button } from '@/components/ui/button'
import { Input } from '@/components/ui/input'
import { Checkbox } from '@/components/ui/checkbox'

import { toast } from 'sonner'

import { QueryMacAddressByPrefix, QueryVersion } from '../../wailsjs/go/service/SQLiteHelper'
import { GetLatestVersion } from '../../wailsjs/go/service/UpdateService'

const Panel: React.FC = () => {
  const [macAddress, setMacAddress] = useState('')
  const [prefix, setPrefix] = useState('')
  const [vendorName, setVendorName] = useState('')
  const [isPrivate, setIsPrivate] = useState(false)
  const [blockType, setBlockType] = useState('')
  const [lastUpdate, setLastUpdate] = useState('')
  const [databaseDate, setDatabaseDate] = useState('')

  const [checkDisabled, setCheckDisabled] = useState(false)

  useEffect(() => {
    QueryVersion().then(res => {
      setDatabaseDate(res)
    })
  }, [])

  const handleTextChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setMacAddress(e.target.value)
    let address = e.target.value
      .trim()
      .toUpperCase()
      .replace(/[:\-\.]/g, '')
    if (address.length < 6) {
      return
    }
    address =
      address.substring(0, 2) + ':' + address.substring(2, 4) + ':' + address.substring(4, 6)
    QueryMacAddressByPrefix(address).then(res => {
      if (res === null) {
        setPrefix('Not Found')
        setVendorName('')
        setIsPrivate(false)
        setBlockType('')
        setLastUpdate('')
        return
      }
      setPrefix(res.Prefix)
      setVendorName(res.VendorName)
      setIsPrivate(res.IsPrivate)
      setBlockType(res.BlockType)
      setLastUpdate(res.LastUpdate)
    })
  }

  const handleCheck = () => {
    setCheckDisabled(true)
    GetLatestVersion().then(res => {
      if (res.Version === null) {
        setCheckDisabled(false)
        toast('Check update failed.', {
          description: 'Please check your internet connection.',
          action: {
            label: 'Close',
            onClick: () => {},
          },
        })
      } else {
        setCheckDisabled(false)
        if (res === databaseDate) {
          toast('No new version available', {
            description: '🎉 You are using the latest version.',
            action: {
              label: 'Close',
              onClick: () => {},
            },
          })
        } else {
        }
      }
    })
  }

  return (
    <div className="mx-auto p-6">
      <div className="mb-2 flex items-center">
        <Input
          className="p-1 border h-8"
          value={macAddress}
          onChange={handleTextChange}
          placeholder="Enter MAC Address"
        />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Prefix</label>
        <Input className="w-2/3 p-1 border h-8" value={prefix} readOnly />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Vendor</label>
        <Input className="w-2/3 p-1 border h-8" value={vendorName} readOnly />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Private</label>
        <Checkbox className="w-4 h-4" checked={isPrivate} disabled />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Block Type</label>
        <Input className="w-2/3 p-1 border h-8" value={blockType} readOnly />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Last Update</label>
        <Input className="w-2/3 p-1 border h-8" value={lastUpdate} readOnly />
      </div>
      <div className="flex items-center">
        <label className="w-1/3 select-none">Database</label>
        <Input className="w-5/12 p-1 border h-8" value={databaseDate} readOnly />
        <Button
          className="ml-2 p-1 h-8 w-1/4 select-none"
          onClick={handleCheck}
          disabled={checkDisabled}
        >
          Check
        </Button>
      </div>
    </div>
  )
}

export default Panel
