package main

import (
	"MacFastLookup/service"
	"embed"
	"os"
	"path/filepath"

	"github.com/wailsapp/wails/v2"
	"github.com/wailsapp/wails/v2/pkg/options"
	"github.com/wailsapp/wails/v2/pkg/options/assetserver"
)

//go:embed all:frontend/dist
var assets embed.FS

func main() {
	// Create an instance of the app structure
	app := NewApp()

	// Create service
	dbFileDir := "MacFastLookup"
	userHomeDir, _ := os.UserHomeDir()
	dbFileName := "mac_vendors.sqlite3"
	dbFilePath := filepath.Join(userHomeDir, dbFileDir, dbFileName)
	dbService := service.NewSQLiteHelper(dbFilePath)

	// create update service
	updateService := service.NewUpdateService("https://tools.taurusxin.com/macfastlookup/latest_version.txt")

	// Create application with options
	err := wails.Run(&options.App{
		Title:  "MacFastLookup",
		Width:  450,
		Height: 350,
		AssetServer: &assetserver.Options{
			Assets: assets,
		},
		BackgroundColour: &options.RGBA{R: 27, G: 38, B: 54, A: 1},
		OnStartup:        app.startup,
		Bind: []interface{}{
			app,
			dbService,
			updateService,
		},
	})

	if err != nil {
		println("Error:", err.Error())
	}
}
